import javax.swing.*;
import squint.*;

// MidTap --- A general purpose interface for interacting with
// networks servers using protocols based on sending text message
// through TCP connections.
public class BooleanTap extends GUIManager 
{
    // Change these values to adjust the size of the program's window
    private final int WINDOW_WIDTH  = 750;
    private final int WINDOW_HEIGHT = 600;
   
    // Dimensions of the text are used to display conversation with server
    private final int LOG_HEIGHT = 24, LOG_WIDTH = 60;
    
    // Widths of text fields used to hold server name and port number
    private final int HOST_WIDTH = 15, PORT_WIDTH = 5;
    
    // Width of text field used to enter messages sent
    private final int MESSAGE_WIDTH = 40;
    
    // Used to enter the host to be contacted
    private JTextField host;

    // Used to enter the port number desired
    private JTextField port;
    
    // Used to enter messages to be sent to the server
    private JTextField message;
    
    // Used to display the sequence of message that have been exchanged
    private JTextArea dialogue;
    
    // Button used to initiate a connection
    private JButton connect;
        
    // Button used to terminate a connection
    private JButton disconnect;
        
    // Button used to send a message
    private JButton send;
        
    // Button used to clear the message display
    private JButton clear;
        
    // The connection between the server and this program
    private NetConnection toServer;

    // Whether a connection is active or not
    private boolean connected = false;   
    

    /*
     * Add all user interface components to the display
     */
    public BooleanTap() {
        this.createWindow( WINDOW_WIDTH, WINDOW_HEIGHT );
        
        JPanel myPane;
        
        // add fields for the host name and port number
        myPane = new JPanel();
        myPane.add( new JLabel( "Server name:") );
        host = new JTextField( HOST_WIDTH );
        myPane.add( host );
        contentPane.add( myPane );
        
        myPane = new JPanel();
        myPane.add( new JLabel( "Port Number:" ) );
        port = new JTextField( PORT_WIDTH);
        myPane.add( port );
        contentPane.add( myPane );
        
        // Add the connect and disconnect buttons
        myPane = new JPanel();
        connect = new JButton( "Connect" );
        myPane.add( connect );
        disconnect = new JButton( "Disconnect" );
        myPane.add( disconnect );
        contentPane.add( myPane );
        
        // add field for message
        myPane = new JPanel();
        myPane.add( new JLabel( "Message: " ) );
        message = new JTextField( MESSAGE_WIDTH );
        myPane.add( message );
        send = new JButton( "Send" );
        myPane.add( send );
        contentPane.add( myPane );
        
        // add display area for messages exchanged
        dialogue = new JTextArea( LOG_HEIGHT, LOG_WIDTH );
        dialogue.setEditable( false );
        contentPane.add( new JScrollPane( dialogue) );
        
        clear = new JButton( "Clear" );
        contentPane.add( clear );
        
        // Place buttons in correct initial state
        connect.setEnabled( true );
        disconnect.setEnabled( false );
        send.setEnabled( false );
      }
    
   /*
    * React when one of the three buttons in the program's interface is
    * clicked
    */
    public void buttonClicked( JButton whichButt ) {
        
        if ( whichButt == connect ) {
            // Create a connection to the server
            toServer = new NetConnection( host.getText(), port.getText() );
            toServer.addMessageListener( this ); 
            
            connected = true;
            
            message.requestFocus();
            message.selectAll();

        } else if ( whichButt == disconnect ) {
            // Tell the server we want to disconnect
            toServer.close();
            
            connected = false;
                        
        } else if ( whichButt == clear ) {
            // Clear the log of messages exchanged
            dialogue.setText("");
        } else if ( whichButt == send ) {
            dialogue.append( "-> " + message.getText() + "\n");
            toServer.out.println( message.getText() );
            
            message.requestFocus();
            message.selectAll();
        }

        // Make sure the buttons are in the right state
        connect.setEnabled( ! connected );
        disconnect.setEnabled( connected );
        send.setEnabled( connected ); 
            
    }
    
   /*
    * Send a message when the user presses return
    */
    public void textEntered( JTextField which ) {
        if ( which == message ) {
            send.doClick();
        }
    }
    
   /*
    * When a message arrives, access it and add it to the display
    */
    public void dataAvailable( NetConnection which ) {
         dialogue.append( "<- " + toServer.in.nextLine() + "\n");
    }
    
   /*
    * When the server closes the connection, note the event in the
    * log 
    */
    public void connectionClosed( NetConnection which ) {
        dialogue.append( "-----Connection terminated-----\n" );
        
        connected = false;
        
        connect.setEnabled( ! connected );
        disconnect.setEnabled( connected );
        send.setEnabled( connected ); 
                    
        host.requestFocus();
        host.selectAll();
    }
}
